<?php
/**
 * OpenVCP Webinterface
 * Copyright (C) 2006 Gerrit Wyen <gerrit@t4a.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * @author  Oliver Werner / OpenVCP Team
 * @version 0.4.0.1
 * @package OpenVCP-v0.4
 */

require_once('User.class.php');
require_once('Node.class.php');

class Admin extends User {
	private $DB = null;
	private $privileges = null;
	private $node = array();
	private $nodeLimits = array();

	public function __construct($admin)
	{
		parent::__construct($admin);
		$this->DB =& OpenVCP::call('DB', 'getReference');
		$result = $this->DB->select($this->DB->table_admin_privileges, NULL, NULL, array('adminid' =>$admin['id']));
		$this->privileges = $result[0];
	}

	/**
	 * returns an array with the images of the admin
	 *
	 * @return array $images
	 */
	public function getImages()
	{
		$imageids = $this->DB->joinselect(array($this->DB->table_images, $this->DB->table_admin_images), NULL, array($this->DB->table_images => 0), array(array($this->DB->table_admin_images => 'adminid', parent::$this->userinfo['id']), 'AND', array($this->DB->table_images => 'id', $this->DB->table_admin_images => 'imageid')), array('alias' => 'ASC'));
		$images = null;
		if(is_array($imageids))
			foreach($imageids as $image)
			{
				if($image['alias'] != "")
					$images[$image['alias']] = $image['name'];
				else 
					$images[] = $image['name'];
			}
		return $images;
	}
	
	public function getImagesExtended(){
		$imageids = $this->DB->joinselect(array($this->DB->table_images, $this->DB->table_admin_images), NULL, array($this->DB->table_images => 0), array(array($this->DB->table_admin_images => 'adminid', parent::$this->userinfo['id']), 'AND', array($this->DB->table_images => 'id', $this->DB->table_admin_images => 'imageid')), array('alias' => 'ASC'));
		$images = null;
		$i = 0;
		if(is_array($imageids))
			foreach($imageids as $image)
			{
				$images[$i]['name'] = $image['name'];
				$images[$i]['alias'] = $image['alias'];
				$images[$i]['text'] = $image['text'];
				$i++;
			}
		return $images;
	}

	public function getNodes()
	{
		return $this->DB->joinselect(array($this->DB->table_nodes, $this->DB->table_admin_nodes), NULL, array($this->DB->table_nodes => 0), array(array( $this->DB->table_admin_nodes => 'nodeid', $this->DB->table_nodes => 'id'), 'AND', array($this->DB->table_admin_nodes => 'adminid', parent::$this->userinfo['id'])), array('name'=>'ASC'));
	}

	public function getVserverCount($nodeid)
	{
		$vservers = $this->DB->select($this->DB->table_vservers, NULL, NULL, array('adminid' => parent::$this->userinfo['id'], 'AND', 'nodeid' => $nodeid));
		return count($vservers);
	}

	/**
	 * get's the admin's settings or return the global setting
	 *
	 * @param string $key
	 * @return string setting
	 */
	public function getSetting($key)
	{
		$settings = $this->DB->select($this->DB->table_admin_settings, NULL, NULL, array('adminid' => $this->userinfo['id'], 'AND', 'key' => $key));
		if(is_array($settings[0]) && $settings[0]['value'] != "")
		{
			return $settings[0]['value'];
		}
		else
		{
			$settings = $this->DB->select($this->DB->table_settings, NULL, NULL, array('key' => $key));
			return (is_array($settings) && $settings[0]['value'] != "") ? $settings[0]['value'] : false;
		}
	}
	
	/**
	 * sets a new setting for the admin
	 * 
	 * @param string $key
	 * @param string $value
	 */
	public function setSetting($key, $value) {
		$setting = $this->DB->select($this->DB->table_admin_settings, NULL, NULL, array('adminid' => $this->userinfo['id'], 'AND', 'key' => $key));
		if(is_array($setting) && $setting[0]['key'] == $key)
		{
			$this->DB->update($this->DB->table_admin_settings, array('value' => $value), array('adminid' => $this->userinfo['id'], 'AND', 'key' => $key));
		}
		else
			$this->DB->update($this->DB->table_settings, array('value' => $value), array('key' => $key));		
		return 1;
	}

	public function getPrivileges($request)
	{
		return $this->privileges[$request];
	}

	private function _loadNodeLimits($nodeid) {
		$limits = $this->DB->select($this->DB->table_admin_nodes, NULL, NULL, array('adminid' => parent::$this->userinfo['id'], 'AND', 'nodeid' => $nodeid));
		$this->nodeLimits[$nodeid] = $limits[0];
		return true;
	}

	public function getLimit($nodeid, $limit, $type = "")
	{
		if(!is_array($this->nodeLimits[$nodeid]))
			$this->_loadNodeLimits($nodeid);
		return $this->nodeLimits[$nodeid][($type != "" ? $limit."_".$type : $limit)."_limit"];
	}

	public function setLimit($nodeid, $limit, $type = "", $value)
	{
		if(!is_array($this->nodeLimits[$nodeid]))
			$this->_loadNodeLimits($nodeid);
		$this->nodeLimits[$nodeid][($type != "" ? $limit."_".$type : $limit)."_limit"] = $value;
		$this->DB->update($this->DB->table_admin_nodes, array(($type != "" ? $limit."_".$type : $limit)."_limit" => $value), array('nodeid' => $nodeid, 'AND', 'adminid' => parent::$this->userinfo['id']));
		return $this->nodeLimits[$nodeid][($type != "" ? $limit."_".$type : $limit)."_limit"];
	}

	public function createURL()
	{
		return "super/admins/admin/".parent::$this->userinfo['loginname'];
	}

}

?>
