<?php
/**
 * OpenVCP Webinterface
 * Copyright (C) 2006 Gerrit Wyen <gerrit@t4a.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * @author  Oliver Werner / OpenVCP Team
 * @version 0.4.0.1
 * @package OpenVCP-v0.4
 */

require_once 'Admin.class.php';

final class AdminFactory {
	
	private $admins = array();
	private $adminNameToID = array();
	private $adminObjects = null;
	private $DB = null;
	static private $object = null;
		
	private function __construct() {
		$this->DB =& OpenVCP::call('DB', 'getReference');
		$adminList = $this->DB->select($this->DB->table_admin);
		foreach($adminList as $admin) {
			$this->admins[$admin['id']] = $admin;
			$this->adminNameToID[$admin['loginname']] = $admin['id'];
		}
	}
	
	static public function getInstance() {
		if(!isset(self::$object)) {
			self::$object = new AdminFactory();
		}
		return self::$object;
	}
	
	public function getUserByName($username) {
		return $this->_getCustomer($this->adminNameToID[$username]);
	}

	public function getUserByID($id) {
		return $this->_getCustomer($id);
	}
	
	/**
	 * return a list of all admins
	 */
	public function getAdminList()
	{
		return $this->admins;
	}
	
	
	private function _getCustomer($id) {
		if($this->adminObjects[$id] == null) {
			$this->adminObjects[$id] = new Admin($this->admins[$id]);
		}
		return $this->adminObjects[$id];
	}
}

?>