<?php
/**
 * OpenVCP Webinterface
 * Copyright (C) 2006 Gerrit Wyen <gerrit@t4a.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * @author  Oliver Werner / OpenVCP Team
 * @version 0.3.0.1
 * @package OpenVCP-v0.3
 */

require_once("Node.class.php");

final Class NodeFactory{
	
	static private $object = null;
	private $nodeObjects = null;
	private $DB = null;
	private $nodeList = null;
	private $nodeNameToID = null;
	
	private function __construct() {
		$this->DB =& OpenVCP::call('DB', 'getReference');
		$nodeList = $this->DB->select($this->DB->table_nodes);
		foreach($nodeList as $node) {
			$this->nodeNameToID[$node['name']] = $node['id'];
			$this->nodeList[$node['id']] = $node;
		}
	}
	
	static public function getNodeFactory() {
		if(!isset(self::$object)) {
			self::$object = new NodeFactory();
		}
		return self::$object;
	}
	
	public function getNodeByName($nodename) {
		return $this->getNodeById($this->nodeNameToID[$nodename]);
	}
	
	public function getNodeById($nodeid) {
		if(!isset($this->nodeObjects[$nodeid])) {
			$this->_getNode($nodeid);
		}
		return $this->nodeObjects[$nodeid];
	}
	
	/**
	 * returns a array containing all nodes
	 *
	 * @return array id and name of all nodes
	 */
	public function getNodeList() {
		return $this->nodeList;
	}
	
	private function _getNode($id) {
		$this->nodeObjects[$id] = new Node($this->nodeList[$id]);
	}
	
	
}

?>