<?php
/**
 * OpenVCP Webinterface
 * Copyright (C) 2006 Gerrit Wyen <gerrit@t4a.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * @author  Oliver Werner / OpenVCP Team
 * @version 0.4.0.1
 * @package OpenVCP-v0.4
 */

require_once 'Vserver.class.php';

final class VserverFactory{
	
	private $vserverObjects = null;
	private $DB = null;
	private $vserverList = null;
	private $nodeAndNameToID = null;
	private $adminAndNameToID = null;
	private $customerAndNameToID = null;
	private $idToAdmin = null;
	static private $object = null;
	
	private function __construct() {
		$this->DB =& OpenVCP::call('DB', 'getReference');
		$vserverList = $this->DB->select($this->DB->table_vservers, NULL, NULL, NULL, array('name' => 'ASC'));
		foreach($vserverList as $vserver) {
			$this->vserverList[$vserver['id']] = $vserver;
			$this->nodeAndNameToID[$vserver['nodeid']][$vserver['name']] = $vserver['id'];
			$this->adminAndNameToID[$vserver['adminid']][$vserver['name']] = $vserver['id'];
			$this->customerAndNameToID[$vserver['userid']][$vserver['name']] = $vserver['id'];
			$this->idToAdmin[$vserver['id']] = $vserver['adminid'];
		}
	}
	
	static public function getInstance() {
		if(!isset(self::$object)) {
			self::$object = new VserverFactory();
		}
		return self::$object;
	}

	public function getVserverByNameAndNode($vservername, $nodeid) {
		return $this->getVserverByID($this->nodeAndNameToID[$nodeid][$vservername]);
	}
	
	public function getVserverByNameAndAdmin($vservername, $adminid) {
		return $this->getVserverByID($this->adminAndNameToID[$adminid][$vservername]);
	}
	
	public function getVserverByNameAndUser($vservername, $userid) {
		return $this->getVserverByID($this->customerAndNameToID[$userid][$vservername]);
	}
	
	public function getVserverByID($vserverid) {
		if(!isset($this->vserverObjects[$vserverid])){
			$this->_getVServer($vserverid);
		}
		return $this->vserverObjects[$vserverid];
	}

	private function _getVServer($id) {
		$this->vserverObjects[$id] = new VServer($this->vserverList[$id]);
	}
	
	/**
	 * Returns a list of all vservers an admin
	 * can control
	 *
	 * @param Admin $admin
	 * @return array of Vserver Objects
	 */
	public function getVserversOfAdmin(Admin $admin) {

		$vservers = $this->vserverList;
		$vserverarray = array();
		foreach($vservers as $vserver) {
			if($admin->getPrivileges('see_all_vserver') == 'y' || $this->idToAdmin[$vserver['id']] == $admin->getId())
				$vserverarray[] = $this->getVserverByID($vserver['id']);
		}
		return $vserverarray;
	}

	/**
	 * returns an array with all vservers as objects
	 * of an specified user
	 *
	 * @param int $userid
	 * @return array of Vserver Objects
	 */
	public function getVServersOfUser($userid) {
		$vservers = $this->customerAndNameToID[$userid];
		$vserverarray = array();
		foreach($vservers as $id) {
			$vserverarray[] = $this->getVserverByID($id);
		}
		return $vserverarray;
	}
	
}

?>
