<?php
/**
 * OpenVCP Webinterface
 * Copyright (C) 2006 Gerrit Wyen <gerrit@t4a.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * @author  Oliver Werner / OpenVCP Team
 * @version 0.4.0.1
 * @package OpenVCP-v0.4
 */
$AUTH   =& OpenVCP::call('Auth', 'getReference');
$GUI    =& OpenVCP::call('GUI' , 'getReference');
$DB		=& OpenVCP::call('DB'  , 'getReference');
$XML    =& OpenVCP::call('XML', 'getReference');
$PARAMS =& OpenVCP::getParams();
$USERINFO = $AUTH->user_information;

require_once('VserverFactory.class.php');
require_once('NodeFactory.class.php');

$nodename = $PARAMS[0];
$vservername = $PARAMS[1];
$node = NodeFactory::getNodeFactory()->getNodeByName($nodename);

$vserver = VserverFactory::getInstance()->getVserverByNameAndNode($vservername, $node->getId());

require_once("navi.php");

$GUI->title(_("Options of vserver")." ".$vserver->getName());
if(isset($_POST['submitlimits']))
{
	if($_POST['schedule'] != "0" && $_POST['schedule'] != "")
	{
		$_POST['schedule'] = trim($_POST['schedule'], " %");
		$_POST['schedule'] = 32*$_POST['schedule']/100;
		$vserver->setSchedule($_POST['schedule']);
	}

	foreach($_POST['rlimit'] as $key => $rlimit)
	{
		foreach($rlimit as $type => $value)
		{
			if($key == "rss" || $key == "as")
			{
				$value = $value * 1024 / 4;
			}
			if($value != "")
			{
				$vserver->setLimit($key.".".$type, $value);
			}
		}
	}
	if($_POST['total'] != $_POST['total_old'] || $_POST['inodes_total'] != $_POST['inodes_total_old'] || $_POST['reserved'] != $_POST['reserved_old'])
	{
		if(!is_numeric($_POST['total']) || $_POST['total'] < 0)
			$_POST['total'] = 0;
		if(!is_numeric($_POST['inodes_total']))
			$_POST['inodes_total'] = 0;
		$vserver->setSpace($_POST['total']*1024, $_POST['inodes_total'], $_POST['reserved']);
	}
	
	if($_POST['trafficlimit_limit_hard'] != "" && $_POST['trafficlimit_limit_hard_orig'] != $_POST['trafficlimit_limit_hard']) {
		$vserver->setTrafficLimit($_POST['trafficlimit_limit_hard'], $_POST['trafficlimit_email_hard'], 'hard');
	}
	else if($_POST['trafficlimit_limit_hard_orig'] != $_POST['trafficlimit_limit_hard'] && $_POST['trafficlimit_limit_hard'] == "") {
		$vserver->delTrafficLimit('hard');
	}
	
	if($_POST['trafficlimit_limit_soft'] != "" && $_POST['trafficlimit_limit_soft_orig'] != $_POST['trafficlimit_limit_soft']) {
		$vserver->setTrafficLimit($_POST['trafficlimit_limit_soft'], $_POST['trafficlimit_email_soft'], 'soft');
	}
	else if($_POST['trafficlimit_limit_soft_orig'] != $_POST['trafficlimit_limit_soft'] && $_POST['trafficlimit_limit_soft'] == "") {
		$vserver->delTrafficLimit('soft');
	}
	
	$GUI->p(_("limits succesfully set. Make sure to restart the vserver before the changes take affect."));
}
$f = $GUI->fieldset("limits");
$form = $GUI->form($f, "limits", (@$PARAMS[2] == "submit" ? "submit" : $PARAMS[1]."/submit"));
$GUI->h(_("scheduling"), 2, $form);
$table = $GUI->table($form);

$schedule_raw = $vserver->getSchedule();
if($schedule_raw['fill-rate'] == "")
{
	$schedule = "";
}
else
{
	$schedule = $schedule_raw['fill-rate']/$schedule_raw['interval']*100;
	$schedule .= " %";
}

$row = $GUI->tableAddRow($table);
$GUI->tableAddCol($row, _("cpu time limit")."[%]:");
$GUI->formAddInputElement($GUI->tableAddCol($row, ""), "text", "schedule", $schedule);

$space =  $vserver->getSpace();

if($space['total'] == -1)
$space['total'] = "N/A";

$GUI->br($form);
$GUI->h(_("space"), 2, $form);
$table = $GUI->table($form);
$thead = $GUI->tableAddHead($table);
$row = $GUI->tableAddRow($thead);
$GUI->tableAddCol($row, _("space total"));
$GUI->tableAddCol($row, _("space used"));
$GUI->tableAddCol($row, _("inodes total"));
$GUI->tableAddCol($row, _("inodes used"));
$GUI->tableAddCol($row, _("reserved total"));
$row = $GUI->tableAddRow($table);
$GUI->formAddInputElement($GUI->tableAddCol($row, ""), "text", "total", ($space['total'] == 0 ? "N/A" : number_format($space['total']/1024, 0, ',', '')));
$GUI->formAddInputElement($form, "hidden", "total_old", ($space['total'] == 0 ? -1 : number_format($space['total']/1024, 0, ',', '')));
$GUI->tableAddCol($row, ($space['used'] == -1 ? "N/A" : number_format($space['used']/1024, 2, ',', '')));
$GUI->formAddInputElement($GUI->tableAddCol($row, ""), "text", "inodes_total", $space['inodes_total']);
$GUI->formAddInputElement($form, "hidden", "inodes_total_old", $space['inodes_total']);
$GUI->tableAddCol($row, ($space['inodes_used'] == -1 ? "N/A" : $space['inodes_used']));
$GUI->formAddInputElement($GUI->tableAddCol($row, ""), "text", "reserved", $space['reserved']);
$GUI->formAddInputElement($form, "hidden", "reserved_old", $space['reserved']);

$response = $vserver->getLimit();

$GUI->br($form);
$GUI->h(_("rlimits"), 2, $form);
$table = $GUI->table($form);
$thead = $GUI->tableAddHead($table);
$row = $GUI->tableAddRow($thead);
$GUI->tableAddCol($row, _("limit"));
$GUI->tableAddCol($row, _("minimum"));
$GUI->tableAddCol($row, _("soft"));
$GUI->tableAddCol($row, _("hard"));
while(($rlimit = $XML->getNextStruct($response)) != null) {
	$row = $GUI->tableAddRow($table);
	$GUI->tableAddCol($row, _($rlimit['name']));
	$GUI->formAddInputElement($GUI->tableAddCol($row, ""), "text", "rlimit[".$rlimit['name']."][min]", ((($rlimit['name'] == "rss" || $rlimit['name'] == "as") &&  $rlimit['min'] != "")? ($rlimit['min'] / 1024 * 4) : $rlimit['min']));
	$GUI->formAddInputElement($GUI->tableAddCol($row, ""), "text", "rlimit[".$rlimit['name']."][soft]", ((($rlimit['name'] == "rss" || $rlimit['name'] == "as") && $rlimit['soft']!= "") ? ($rlimit['soft'] / 1024 * 4) : $rlimit['soft']));
	$GUI->formAddInputElement($GUI->tableAddCol($row, ""), "text", "rlimit[".$rlimit['name']."][hard]", ((($rlimit['name'] == "rss" || $rlimit['name'] == "as") && $rlimit['hard'] != "") ? ($rlimit['hard'] / 1024 * 4) : $rlimit['hard']));
}

$trafficlimits = $vserver->getTrafficLimit();
while($limit = $XML->getNextStruct($trafficlimits)) {
	if($limit['type'] == 'soft')
		$softlimits = $limit;
	if($limit['type'] == 'hard')
		$hardlimits = $limit;
}

$GUI->br($form);
$GUI->h(_("hard traffic limits"), 2, $form);
$table = $GUI->table($form);
$thead = $GUI->tableAddHead($table);
$row = $GUI->tableAddRow($thead);
$GUI->tableAddCol($row, _("type"));
$GUI->tableAddCol($row, _("limit")." [MB]");
$GUI->tableAddCol($row, _("email for notification"));
$row = $GUI->tableAddRow($table);
$GUI->tableAddCol($row, _("soft"));
$GUI->formAddInputElement($GUI->tableAddCol($row, ""), "text", "trafficlimit_limit_soft", $softlimits['total']);
$GUI->formAddInputElement($GUI->tableAddCol($row, ""), "text", "trafficlimit_email_soft", $softlimits['notify']);
$row = $GUI->tableAddRow($table);
$GUI->tableAddCol($row, _("hard"));
$GUI->formAddInputElement($GUI->tableAddCol($row, ""), "text", "trafficlimit_limit_hard", $hardlimits['total']);
$GUI->formAddInputElement($GUI->tableAddCol($row, ""), "text", "trafficlimit_email_hard", $hardlimits['notify']);

$GUI->formAddInputElement($form, "hidden", "trafficlimit_limit_soft_orig", $softlimits['total']);
$GUI->formAddInputElement($form, "hidden", "trafficlimit_limit_hard_orig", $hardlimits['total']);
$GUI->formAddInputElement($form, "hidden", "trafficlimit_email_orig", $hardlimits['notify']);

$GUI->formAddInputElement($form, "submit", "submitlimits", _("save"));
?>
